@@ -170,6 +170,7 @@ def fit_ml(
     # encoder
     only_categorical: bool = False,
     encoder_config: Optional[Dict[str, Any]] = None,
+    categorical_dims: Optional[Dict[str, int]] = None,
     encoding_methods: Optional[Dict[str, List[str]]] = None,
     encoding_configs: Optional[Dict[str, Dict[str, Any]]] = None,
     default_encoding_methods: Optional[List[str]] = None,
@@ -221,6 +222,7 @@ def fit_ml(
         loss_config=loss_config,
         only_categorical=only_categorical,
         encoder_config=encoder_config,
+        categorical_dims=categorical_dims,
         encoding_methods=encoding_methods,
         encoding_configs=encoding_configs,
         default_encoding_methods=default_encoding_methods,
@@ -294,6 +296,7 @@ def repeat_ml(
     available_cuda_list: Optional[List[int]] = None,
     resource_config: Optional[Dict[str, Any]] = None,
     task_meta_kwargs: Optional[Dict[str, Any]] = None,
+    to_original_device: bool = False,
     is_fix: bool = False,
     # data
     carefree: bool = False,
@@ -344,6 +347,7 @@ def repeat_ml(
     finetune_config: Optional[Dict[str, Any]] = None,
     tqdm_settings: Optional[Dict[str, Any]] = None,
     # misc
+    sample_weights: sample_weights_type = None,
     pre_process_batch: bool = True,
     debug: bool = False,
 ) -> RepeatResult:
@@ -403,11 +407,12 @@ def repeat_ml(
             data_config,
             cf_data_config,
         ),
-        pipeline_base=MLCarefree,
+        carefree=carefree,
         workplace=workplace,
         models=models,
         model_configs=model_configs,
         predict_config=predict_config,
+        sample_weights=sample_weights,
         sequential=sequential,
         num_jobs=num_jobs,
         num_repeat=num_repeat,
@@ -417,6 +422,7 @@ def repeat_ml(
         available_cuda_list=available_cuda_list,
         resource_config=resource_config,
         task_meta_kwargs=task_meta_kwargs,
+        to_original_device=to_original_device,
         is_fix=is_fix,
         **pipeline_config,
     )
