@@ -1,7 +1,6 @@
 import os
 
 import numpy as np
-import matplotlib.pyplot as plt
 
 from typing import Any
 from typing import List
@@ -9,23 +8,30 @@ from typing import Tuple
 from typing import Union
 from typing import Callable
 from typing import Optional
-from cfml.misc.toolkit import show_or_save
 
+from ..pipeline import SimplePipeline
 from ....types import tensor_dict_type
-from ....constants import INPUT_KEY
 from ....constants import PREDICTIONS_KEY
 from ....misc.toolkit import to_numpy
 from ....misc.toolkit import to_torch
 from ....models.ml import DDR
 
+try:
+    import matplotlib.pyplot as plt
+    from cfml.misc.toolkit import show_or_save
+
+    Figure = plt.Figure
+except:
+    plt = show_or_save = Figure = None
+
 
 class DDRPredictor:
     def __init__(self, ddr: DDR):
         self.m = ddr
 
     def _fetch(self, x: np.ndarray, **kwargs: Any) -> tensor_dict_type:
-        x_tensor = to_torch(x).to(self.m.device)
-        return self.m(0, {INPUT_KEY: x_tensor}, **kwargs)
+        net = to_torch(x).to(self.m.device)
+        return self.m(net, **kwargs)
 
     def median(self, x: np.ndarray) -> np.ndarray:
         results = self._fetch(x, get_quantiles=False)
@@ -47,6 +53,10 @@ class DDRPredictor:
         results = self._fetch(x, y_anchor=y_anchor, get_quantiles=False)
         return to_numpy(results["cdf"]), to_numpy(results["pdf"])
 
+    @classmethod
+    def from_pipeline(cls, m: SimplePipeline) -> "DDRPredictor":
+        return cls(m.model.core.core)
+
 
 class DDRVisualizer:
     def __init__(
@@ -55,6 +65,8 @@ class DDRVisualizer:
         dpi: int = 200,
         figsize: Tuple[int, int] = (8, 6),
     ):
+        if show_or_save is None:
+            raise ValueError("`carefree-ml` is needed for `DDRVisualizer`")
         self.m = ddr
         self.dpi = dpi
         self.figsize = figsize
@@ -69,7 +81,7 @@ class DDRVisualizer:
         median: Optional[np.ndarray],
         indices: np.ndarray,
         title: str,
-    ) -> plt.Figure:
+    ) -> Figure:  # type: ignore
         figure = plt.figure(figsize=self.figsize, dpi=self.dpi)
         plt.title(title)
         plt.scatter(x[indices], y[indices], color="gray", s=15)
@@ -206,6 +218,15 @@ class DDRVisualizer:
                 _plot("cdf", anchor, yd, cdf, anchor_line)
                 _plot("pdf", anchor, None, pdf, anchor_line)
 
+    @classmethod
+    def from_pipeline(
+        cls,
+        m: SimplePipeline,
+        dpi: int = 200,
+        figsize: Tuple[int, int] = (8, 6),
+    ) -> "DDRVisualizer":
+        return cls(m.model.core.core, dpi, figsize)
+
 
 __all__ = [
     "DDRPredictor",
