@@ -12,14 +12,12 @@ from typing import Callable
 from typing import Optional
 from functools import partial
 from collections import OrderedDict
-from cfdata.tabular import ColumnTypes
 from cftool.misc import get_arguments
 from cftool.misc import lock_manager
 from cftool.misc import Saving
 from cftool.array import softmax
 from cftool.array import squeeze
 from cftool.array import is_float
-from cfml.misc.toolkit import ModelPattern
 
 from ...data import MLData
 from ...data import MLLoader
@@ -33,10 +31,19 @@ from ...pipeline import DLPipeline
 from ...protocol import InferenceOutputs
 from ...constants import PT_PREFIX
 from ...constants import SCORES_FILE
+from ...constants import WARNING_PREFIX
 from ...constants import PREDICTIONS_KEY
-from ...misc.internal_ import MLInference
+from ...misc.toolkit import get_full_logits
+from ...misc.toolkit import get_label_predictions
 from ...models.ml.encoders import Encoder
 from ...models.ml.protocol import MLModel
+from ...misc.internal_.inference import MLInference
+
+try:
+    from cfdata.tabular import ColumnTypes
+    from cfml.misc.toolkit import ModelPattern
+except:
+    ColumnTypes = ModelPattern = None
 
 
 @DLPipeline.register("ml.simple")
@@ -63,6 +70,7 @@ class SimplePipeline(DLPipeline):
         # encoder
         only_categorical: bool = False,
         encoder_config: Optional[Dict[str, Any]] = None,
+        categorical_dims: Optional[Dict[str, int]] = None,
         encoding_methods: Optional[Dict[str, List[str]]] = None,
         encoding_configs: Optional[Dict[str, Dict[str, Any]]] = None,
         default_encoding_methods: Optional[List[str]] = None,
@@ -146,6 +154,7 @@ class SimplePipeline(DLPipeline):
         self.only_categorical = only_categorical
         self.encoder = None
         self.encoder_config = encoder_config or {}
+        self.categorical_dims = categorical_dims or {}
         self.encoding_methods = encoding_methods or {}
         self.encoding_configs = encoding_configs or {}
         if default_encoding_methods is None:
@@ -169,15 +178,88 @@ class SimplePipeline(DLPipeline):
                 )
         self.use_auto_loss = self.loss_name == "auto"
         if self.use_auto_loss:
-            self.loss_name = "mae" if is_reg else "focal"
-        if self.encoder is None:
-            self.numerical_columns_mapping = {i: i for i in range(self.input_dim)}
+            if is_reg:
+                self.loss_name = "mae"
+            else:
+                self.loss_name = "bce" if self.output_dim == 1 else "focal"
+
+    def _instantiate_encoder(
+        self,
+        categorical_dims: List[int],
+        encoding_methods: List[List[str]],
+        encoding_configs: List[Dict[str, Any]],
+        categorical_columns: List[int],
+    ) -> Encoder:
+        if self.in_loading:
+            loaders = []
+        else:
+            train_loader, valid_loader = self.data.initialize()
+            assert isinstance(train_loader, MLLoader)
+            train_loader_copy = train_loader.copy()
+            train_loader_copy.disable_shuffle()
+            loaders = [train_loader_copy]
+            if valid_loader is not None:
+                assert isinstance(valid_loader, MLLoader)
+                loaders.append(valid_loader)
+        return Encoder(
+            self.encoder_config,
+            categorical_dims,
+            encoding_methods,  # type: ignore
+            encoding_configs,
+            categorical_columns,
+            loaders,
+        )
+
+    def _setup_encoder(self, data_info: Dict[str, Any]) -> None:
+        categorical_dims = []
+        encoding_methods = []
+        encoding_configs = []
+        categorical_columns = []
+        use_one_hot = False
+        use_embedding = False
+        for str_idx, categorical_dim in self.categorical_dims.items():
+            idx_encoding_methods = self.encoding_methods.setdefault(
+                str_idx,
+                self.default_encoding_methods,
+            )
+            idx_encoding_configs = self.encoding_configs.setdefault(
+                str_idx,
+                self.default_encoding_configs,
+            )
+            if isinstance(idx_encoding_methods, str):
+                idx_encoding_methods = [idx_encoding_methods]
+            use_one_hot = use_one_hot or "one_hot" in idx_encoding_methods
+            use_embedding = use_embedding or "embedding" in idx_encoding_methods
+            categorical_dims.append(categorical_dim)
+            encoding_methods.append(idx_encoding_methods)
+            encoding_configs.append(idx_encoding_configs)
+            categorical_columns.append(int(str_idx))
+
+        assert isinstance(self.input_dim, int)
+        all_indices = list(range(self.input_dim))
+        if not categorical_columns:
+            self.encoder = None
+            self.numerical_columns_mapping = {i: i for i in all_indices}
             self.categorical_columns_mapping = {}
             self.use_one_hot = False
             self.use_embedding = False
+        else:
+            self.encoder = self._instantiate_encoder(
+                categorical_dims,
+                encoding_methods,
+                encoding_configs,
+                categorical_columns,
+            )
+            self.use_one_hot = use_one_hot
+            self.use_embedding = use_embedding
+            categorical_set = set(categorical_columns)
+            numerical_columns = [i for i in all_indices if i not in categorical_set]
+            self.numerical_columns_mapping = {i: i for i in numerical_columns}
+            self.categorical_columns_mapping = {i: i for i in categorical_columns}
 
     def _prepare_modules(self, data_info: Dict[str, Any]) -> None:
         self._setup_defaults(data_info)
+        self._setup_encoder(data_info)
         self._prepare_workplace()
         self._prepare_loss()
         assert isinstance(self.input_dim, int)
@@ -222,15 +304,19 @@ class SimplePipeline(DLPipeline):
     def to_pattern(
         self,
         *,
+        binary_threshold: float = 0.5,
         pre_process: Optional[Callable] = None,
         **predict_kwargs: Any,
     ) -> ModelPattern:
+        if ModelPattern is None:
+            raise ValueError("`carefree-ml` need to be installed to use `to_pattern`")
+
         def _predict(x: np.ndarray) -> np.ndarray:
             if pre_process is not None:
                 x = pre_process(x)
             predictions = self.predict(x, **predict_kwargs)[PREDICTIONS_KEY]
             if self.is_classification:
-                return np.argmax(predictions, axis=1)[..., None]
+                return get_label_predictions(predictions, binary_threshold)
             return predictions
 
         def _predict_prob(x: np.ndarray) -> np.ndarray:
@@ -240,6 +326,7 @@ class SimplePipeline(DLPipeline):
             if pre_process is not None:
                 x = pre_process(x)
             logits = self.predict(x, **predict_kwargs)[PREDICTIONS_KEY]
+            logits = get_full_logits(logits)
             return softmax(logits)
 
         return ModelPattern(
@@ -266,11 +353,12 @@ class SimplePipeline(DLPipeline):
                 m = cls._load_infrastructure(
                     export_folder,
                     cuda,
+                    False,
                     pre_callback,
                     post_callback,
                 )
                 assert isinstance(m, SimplePipeline)
-                data_info = DLDataModule.load(export_folder)
+                data_info = DLDataModule.load_info(export_folder)
         m._num_repeat = m.config["num_repeat"] = len(export_folders)
         m._prepare_modules(data_info)
         m.model.to(m.device)
@@ -308,6 +396,17 @@ class SimplePipeline(DLPipeline):
                 Saving.compress(abs_folder, remove_original=True)
         return self
 
+    @classmethod
+    def _load_states_callback(cls, m: Any, states: Dict[str, Any]) -> Dict[str, Any]:
+        if m.encoder is not None:
+            encoder_cache_keys = []
+            for key in states:
+                if key.startswith("encoder") and key.endswith("cache"):
+                    encoder_cache_keys.append(key)
+            for key in encoder_cache_keys:
+                states.pop(key)
+        return states
+
 
 @DLPipeline.register("ml.carefree")
 class CarefreePipeline(SimplePipeline):
@@ -323,6 +422,7 @@ class CarefreePipeline(SimplePipeline):
         # encoder
         only_categorical: bool = False,
         encoder_config: Optional[Dict[str, Any]] = None,
+        categorical_dims: Optional[Dict[str, int]] = None,
         encoding_methods: Optional[Dict[str, List[str]]] = None,
         encoding_configs: Optional[Dict[str, Dict[str, Any]]] = None,
         default_encoding_methods: Optional[List[str]] = None,
@@ -363,6 +463,11 @@ class CarefreePipeline(SimplePipeline):
         pre_process_batch: bool = True,
         num_repeat: Optional[int] = None,
     ):
+        if ColumnTypes is None:
+            raise ValueError(
+                "`carefree-data` need to be installed for `ml.CarefreePipeline`, "
+                "please use `ml.SimplePipeline` instead."
+            )
         config = get_arguments()
         super().__init__(
             core_name,
@@ -373,6 +478,7 @@ class CarefreePipeline(SimplePipeline):
             loss_config=loss_config,
             only_categorical=only_categorical,
             encoder_config=encoder_config,
+            categorical_dims=categorical_dims,
             encoding_methods=encoding_methods,
             encoding_configs=encoding_configs,
             default_encoding_methods=default_encoding_methods,
@@ -413,8 +519,11 @@ class CarefreePipeline(SimplePipeline):
         )
         self.config = config
 
-    def _prepare_modules(self, data_info: Dict[str, Any]) -> None:
+    def _setup_encoder(self, data_info: Dict[str, Any]) -> None:
         self.cf_data = data_info["cf_data"]
+        if self.cf_data is None:
+            msg = "cf_data` is not provided, please use `ml.SimplePipeline` instead"
+            raise ValueError(msg)
         # encoder
         excluded = 0
         numerical_columns_mapping = {}
@@ -422,7 +531,7 @@ class CarefreePipeline(SimplePipeline):
         categorical_dims = []
         encoding_methods = []
         encoding_configs = []
-        true_categorical_columns = []
+        categorical_columns = []
         use_one_hot = False
         use_embedding = False
         if self.cf_data.is_simplify:
@@ -441,7 +550,11 @@ class CarefreePipeline(SimplePipeline):
                     numerical_columns_mapping[idx] = idx - excluded
                 else:
                     str_idx = str(idx)
-                    categorical_dims.append(recognizer.num_unique_values)
+                    categorical_dim = self.categorical_dims.setdefault(
+                        str_idx,
+                        recognizer.num_unique_values,
+                    )
+                    categorical_dims.append(categorical_dim)
                     idx_encoding_methods = self.encoding_methods.setdefault(
                         str_idx,
                         self.default_encoding_methods,
@@ -458,36 +571,22 @@ class CarefreePipeline(SimplePipeline):
                         )
                     )
                     true_idx = idx - excluded
-                    true_categorical_columns.append(true_idx)
+                    categorical_columns.append(true_idx)
                     categorical_columns_mapping[idx] = true_idx
-        if not true_categorical_columns:
+        if not categorical_columns:
             encoder = None
         else:
-            if self.in_loading:
-                loaders = []
-            else:
-                train_loader, valid_loader = self.data.initialize()
-                assert isinstance(train_loader, MLLoader)
-                train_loader_copy = train_loader.copy()
-                train_loader_copy.disable_shuffle()
-                loaders = [train_loader_copy]
-                if valid_loader is not None:
-                    assert isinstance(valid_loader, MLLoader)
-                    loaders.append(valid_loader)
-            encoder = Encoder(
-                self.encoder_config,
+            encoder = self._instantiate_encoder(
                 categorical_dims,
-                encoding_methods,  # type: ignore
+                encoding_methods,
                 encoding_configs,
-                true_categorical_columns,
-                loaders,
+                categorical_columns,
             )
         self.encoder = encoder
         self.use_one_hot = use_one_hot
         self.use_embedding = use_embedding
         self.numerical_columns_mapping = numerical_columns_mapping
         self.categorical_columns_mapping = categorical_columns_mapping
-        super()._prepare_modules(data_info)
 
     def _predict_from_outputs(self, outputs: InferenceOutputs) -> np_dict_type:
         assert self.cf_data is not None
@@ -523,30 +622,21 @@ class CarefreePipeline(SimplePipeline):
         cls,
         export_folder: str,
         cuda: Optional[str],
+        to_original_device: bool,
         pre_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
         post_callback: Optional[Callable[[DLPipeline, Dict[str, Any]], None]] = None,
     ) -> "CarefreePipeline":
         m = super()._load_infrastructure(
             export_folder,
             cuda,
+            to_original_device,
             pre_callback,
             post_callback,
         )
         assert isinstance(m, CarefreePipeline)
-        m.cf_data = DLDataModule.load(export_folder)["cf_data"]
+        m.cf_data = DLDataModule.load_info(export_folder)["cf_data"]
         return m
 
-    @classmethod
-    def _load_states_callback(cls, m: Any, states: Dict[str, Any]) -> Dict[str, Any]:
-        if m.encoder is not None:
-            encoder_cache_keys = []
-            for key in states:
-                if key.startswith("encoder") and key.endswith("cache"):
-                    encoder_cache_keys.append(key)
-            for key in encoder_cache_keys:
-                states.pop(key)
-        return states
-
 
 __all__ = [
     "SimplePipeline",
