@@ -1,7 +1,6 @@
 import os
 import dill
 import json
-import lmdb
 import torch
 
 import numpy as np
@@ -45,16 +44,27 @@ from ..misc.toolkit import to_device
 from ..misc.toolkit import get_ddp_info
 from ..misc.toolkit import get_world_size
 
+try:
+    import lmdb
+except:
+    lmdb = None
+
 
 data_modules: Dict[str, Type["DataModule"]] = {}
 DataModuleType = TypeVar("DataModuleType", bound="DataModule", covariant=True)
 
 
+class BaseResponse(NamedTuple):
+    folder: str
+    base: Type["DataModule"]
+
+
 class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
     d = data_modules  # type: ignore
 
     id_file = "id.txt"
     info_name = "info"
+    data_folder = "data"
     package_folder = "data_module"
 
     # inherit
@@ -72,16 +82,37 @@ class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
 
     # internal
 
+    @abstractmethod
+    def _save_data(self, data_folder: str) -> None:
+        pass
+
+    @classmethod
+    @abstractmethod
+    def _load(
+        cls,
+        data_folder: str,
+        info: Dict[str, Any],
+        sample_weights: sample_weights_type,
+    ) -> "DataModule":
+        pass
+
     def _save_info(self, folder: str) -> None:
         Saving.save_dict(self.info, self.info_name, folder)
 
     @classmethod
-    def load_info(cls, folder: str) -> Dict[str, Any]:
+    def _load_info(cls, folder: str) -> Dict[str, Any]:
         return Saving.load_dict(cls.info_name, folder)
 
+    @classmethod
+    def _get_base(cls, folder: str) -> BaseResponse:
+        folder = os.path.join(folder, cls.package_folder)
+        with open(os.path.join(folder, cls.id_file), "r") as f:
+            base = cls.get(f.read())
+        return BaseResponse(folder, base)
+
     # api
 
-    def save(self, folder: str) -> None:
+    def save_info(self, folder: str) -> None:
         folder = os.path.join(folder, self.package_folder)
         os.makedirs(folder, exist_ok=True)
         with open(os.path.join(folder, self.id_file), "w") as f:
@@ -89,39 +120,35 @@ class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
         self._save_info(folder)
 
     @classmethod
-    def load(cls, folder: str) -> Dict[str, Any]:
-        folder = os.path.join(folder, cls.package_folder)
-        with open(os.path.join(folder, cls.id_file), "r") as f:
-            base = cls.get(f.read())
-        return base.load_info(folder)
+    def load_info(cls, folder: str) -> Dict[str, Any]:
+        folder, base = cls._get_base(folder)
+        return base._load_info(folder)
+
+    def save(self, folder: str) -> None:
+        self.save_info(folder)
+        data_folder = os.path.join(folder, self.package_folder, self.data_folder)
+        os.makedirs(data_folder, exist_ok=True)
+        self._save_data(data_folder)
+
+    @classmethod
+    def load(
+        cls,
+        folder: str,
+        *,
+        sample_weights: sample_weights_type = None,
+    ) -> "DataModule":
+        folder, base = cls._get_base(folder)
+        info = base._load_info(folder)
+        data_folder = os.path.join(folder, cls.data_folder)
+        return base._load(data_folder, info, sample_weights)
 
 
 @DataModule.register("dl")  # type: ignore
 class DLDataModule(DataModule, metaclass=ABCMeta):
-    test_transform: Optional[Callable]
-    transform_file = "transform.pkl"
-
     @abstractmethod
     def initialize(self) -> Tuple[DataLoaderProtocol, Optional[DataLoaderProtocol]]:
         pass
 
-    def _save_info(self, folder: str) -> None:
-        super()._save_info(folder)
-        with open(os.path.join(folder, self.transform_file), "wb") as f:
-            dill.dump(self.test_transform, f)
-
-    @classmethod
-    def load_info(cls, folder: str) -> Dict[str, Any]:
-        info = super().load_info(folder)
-        transform_path = os.path.join(folder, cls.transform_file)
-        if not os.path.isfile(transform_path):
-            test_transform = None
-        else:
-            with open(transform_path, "rb") as f:
-                test_transform = dill.load(f)
-        info["test_transform"] = test_transform
-        return info
-
 
 def get_weighted_indices(
     n: int,
@@ -484,7 +511,12 @@ class ImageFolderDataset(Dataset):
                 ),
             )
         )
-        if lmdb_config is None:
+        if lmdb_config is None or lmdb is None:
+            if lmdb_config is not None:
+                print(
+                    f"{WARNING_PREFIX}`lmdb` is not installed, "
+                    "so `lmdb_config` will be ignored"
+                )
             self.lmdb = self.context = None
             with open(os.path.join(self.folder, f"{LABEL_KEY}.json"), "r") as f:
                 self.labels = {LABEL_KEY: json.load(f)}
