@@ -1,7 +1,6 @@
 import os
 import dill
 import json
-import lmdb
 import torch
 import random
 import shutil
@@ -24,10 +23,10 @@ from typing import Optional
 from typing import NamedTuple
 from cftool.dist import Parallel
 from cftool.misc import walk
+from cftool.misc import get_arguments
 from cftool.misc import shallow_copy_dict
 from cftool.misc import Saving
 from torch.utils.data import Dataset
-from cfdata.tabular.api import TabularData
 
 from .core import default_lmdb_path
 from .core import CVLoader
@@ -52,6 +51,15 @@ from ..constants import ERROR_PREFIX
 from ..constants import WARNING_PREFIX
 from ..constants import ORIGINAL_LABEL_KEY
 
+try:
+    import lmdb
+except:
+    lmdb = None
+try:
+    from cfdata.tabular.api import TabularData
+except:
+    TabularData = None
+
 
 # dl
 
@@ -104,6 +112,15 @@ class TensorDictDataset(Dataset):
 
 @DLDataModule.register("tensor")
 class TensorData(DLDataModule):
+    files = [
+        "x_train.pt",
+        "y_train.pt",
+        "x_valid.pt",
+        "y_valid.pt",
+        "train_others.pt",
+        "valid_others.pt",
+    ]
+
     def __init__(
         self,
         x_train: Tensor,
@@ -124,7 +141,6 @@ class TensorData(DLDataModule):
         self.train_others = train_others
         self.valid_others = valid_others
         self.kw = dict(batch_size=batch_size, shuffle=shuffle, num_workers=num_workers)
-        self.test_transform = None
 
     @property
     def info(self) -> Dict[str, Any]:
@@ -149,9 +165,33 @@ class TensorData(DLDataModule):
             valid_loader = DLLoader(DataLoader(self.valid_data, **self.kw))  # type: ignore
         return train_loader, valid_loader
 
+    def _save_data(self, data_folder: str) -> None:
+        all_data = [self.x_train, self.y_train, self.x_valid, self.y_valid]
+        all_data += [self.train_others, self.valid_others]
+        for data, file in zip(all_data, self.files):
+            if data is not None:
+                torch.save(data, os.path.join(data_folder, file))
+
+    @classmethod
+    def _load(
+        cls,
+        data_folder: str,
+        info: Dict[str, Any],
+        sample_weights: sample_weights_type,
+    ) -> "TensorData":
+        args = []
+        for file in cls.files:
+            path = os.path.join(data_folder, file)
+            args.append(None if not os.path.isfile(path) else torch.load(path))
+        data = cls(*args, **info)
+        data.prepare(sample_weights)
+        return data
+
 
 @DLDataModule.register("tensor_dict")
 class TensorDictData(DLDataModule):
+    files = ["x_train.pt", "y_train.pt", "x_valid.pt", "y_valid.pt"]
+
     def __init__(
         self,
         x_train: tensor_dict_type,
@@ -168,7 +208,6 @@ class TensorDictData(DLDataModule):
         self.x_valid = x_valid
         self.y_valid = y_valid
         self.kw = dict(batch_size=batch_size, shuffle=shuffle, num_workers=num_workers)
-        self.test_transform = None
 
     @property
     def info(self) -> Dict[str, Any]:
@@ -193,6 +232,27 @@ class TensorDictData(DLDataModule):
             valid_loader = DLLoader(DataLoader(self.valid_data, **self.kw))  # type: ignore
         return train_loader, valid_loader
 
+    def _save_data(self, data_folder: str) -> None:
+        all_data = [self.x_train, self.y_train, self.x_valid, self.y_valid]
+        for data, file in zip(all_data, self.files):
+            if data is not None:
+                torch.save(data, os.path.join(data_folder, file))
+
+    @classmethod
+    def _load(
+        cls,
+        data_folder: str,
+        info: Dict[str, Any],
+        sample_weights: sample_weights_type,
+    ) -> "TensorData":
+        args = []
+        for file in cls.files:
+            path = os.path.join(data_folder, file)
+            args.append(None if not os.path.isfile(path) else torch.load(path))
+        data = cls(*args, **info)  # type: ignore
+        data.prepare(sample_weights)
+        return data
+
 
 @DLDataModule.register("dummy")
 class DummyData(TensorData):
@@ -237,7 +297,15 @@ class MLData(DLDataModule):
     train_data: MLDataset
     valid_data: Optional[MLDataset]
 
+    data_files = [
+        "x_train.npy",
+        "y_train.npy",
+        "x_valid.npy",
+        "y_valid.npy",
+    ]
+    arguments_file = "arguments.json"
     tmp_cf_data_name = ".tmp_cf_data"
+    full_cf_data_name = "cf_data"
 
     def __init__(
         self,
@@ -264,6 +332,10 @@ class MLData(DLDataModule):
         # inference
         for_inference: bool = False,
     ):
+        self.arguments = shallow_copy_dict(get_arguments())
+        pop_keys = ["x_train", "y_train", "x_valid", "y_valid", "cf_data"]
+        for key in pop_keys:
+            self.arguments.pop(key)
         assert x_train is not None
         self.x_train = x_train
         self.y_train = y_train
@@ -285,6 +357,7 @@ class MLData(DLDataModule):
         self.shuffle_valid = shuffle_valid
         self.batch_size = batch_size
         self.valid_batch_size = valid_batch_size
+        self.loaded = False
 
     @property
     def info(self) -> Dict[str, Any]:
@@ -299,7 +372,8 @@ class MLData(DLDataModule):
     def prepare(self, sample_weights: sample_weights_type) -> None:
         self.train_weights, self.valid_weights = _split_sw(sample_weights)
         if self.cf_data is not None:
-            self.cf_data.read(self.x_train, self.y_train, **self.read_config)
+            if not self.loaded:
+                self.cf_data.read(self.x_train, self.y_train, **self.read_config)
             if self.is_classification is None:
                 self.is_classification = self.cf_data.is_clf
             if self.x_valid is not None:
@@ -364,6 +438,25 @@ class MLData(DLDataModule):
             )
         return train_loader, valid_loader
 
+    @classmethod
+    def with_cf_data(
+        cls,
+        *args: Any,
+        is_classification: Optional[bool] = None,
+        cf_data_config: Optional[Dict[str, Any]] = None,
+        **kwargs: Any,
+    ) -> "MLData":
+        if TabularData is None:
+            raise ValueError("`carefree-data` needs to be installed for `with_cf_data`")
+        if cf_data_config is None:
+            cf_data_config = {}
+        cf_data_config["default_categorical_process"] = "identical"
+        if is_classification is not None:
+            cf_data_config["task_type"] = "clf" if is_classification else "reg"
+        kwargs["is_classification"] = is_classification
+        kwargs["cf_data"] = TabularData(**(cf_data_config or {}))
+        return cls(*args, **kwargs)
+
     def _save_info(self, folder: str) -> None:
         info = self.info
         if info["cf_data"] is not None:
@@ -377,11 +470,16 @@ class MLData(DLDataModule):
         Saving.save_dict(info, self.info_name, folder)
 
     @classmethod
-    def load_info(cls, folder: str) -> Dict[str, Any]:
-        d = super().load_info(folder)
+    def _load_info(cls, folder: str) -> Dict[str, Any]:
+        d = super()._load_info(folder)
         cf_data = d["cf_data"]
         if cf_data is None:
             return d
+        if TabularData is None:
+            raise ValueError(
+                "`carefree-data` needs to be installed "
+                "to load `MLData` with `cf_data` defined"
+            )
         with tempfile.TemporaryDirectory() as tmp_dir:
             tmp_name = os.path.join(tmp_dir, cls.tmp_cf_data_name)
             zip_file = f"{tmp_name}.zip"
@@ -391,22 +489,42 @@ class MLData(DLDataModule):
             os.remove(zip_file)
         return d
 
+    def _save_data(self, data_folder: str) -> None:
+        with open(os.path.join(data_folder, self.arguments_file), "w") as f:
+            json.dump(self.arguments, f)
+        all_data = [self.x_train, self.y_train, self.x_valid, self.y_valid]
+        for data, file in zip(all_data, self.data_files):
+            if data is not None:
+                np.save(os.path.join(data_folder, file), data)
+        if self.cf_data is not None:
+            full_cf_data_folder = os.path.join(data_folder, self.full_cf_data_name)
+            self.cf_data.save(full_cf_data_folder, retain_data=True)
+
     @classmethod
-    def with_cf_data(
+    def _load(
         cls,
-        *args: Any,
-        is_classification: Optional[bool] = None,
-        cf_data_config: Optional[Dict[str, Any]] = None,
-        **kwargs: Any,
+        data_folder: str,
+        info: Dict[str, Any],
+        sample_weights: sample_weights_type,
     ) -> "MLData":
-        if cf_data_config is None:
-            cf_data_config = {}
-        cf_data_config["default_categorical_process"] = "identical"
-        if is_classification is not None:
-            cf_data_config["task_type"] = "clf" if is_classification else "reg"
-        kwargs["is_classification"] = is_classification
-        kwargs["cf_data"] = TabularData(**(cf_data_config or {}))
-        return cls(*args, **kwargs)
+        args = []
+        for file in cls.data_files:
+            path = os.path.join(data_folder, file)
+            args.append(None if not os.path.isfile(path) else np.load(path))
+        with open(os.path.join(data_folder, cls.arguments_file), "r") as f:
+            kwargs = json.load(f)
+        if info["cf_data"] is not None:
+            if TabularData is None:
+                raise ValueError(
+                    "`carefree-data` needs to be installed "
+                    "to load `MLData` with `cf_data` defined"
+                )
+            full_cf_data_folder = os.path.join(data_folder, cls.full_cf_data_name)
+            kwargs["cf_data"] = TabularData.load(full_cf_data_folder)
+        data = cls(*args, **kwargs)
+        data.loaded = True
+        data.prepare(sample_weights)
+        return data
 
 
 class MLInferenceData(MLData):
@@ -418,7 +536,45 @@ class MLInferenceData(MLData):
 
 
 class CVDataModule(DLDataModule, metaclass=ABCMeta):
+    arguments: Dict[str, Any]
     test_transform: Optional[Transforms]
+    transform_file = "transform.pkl"
+    arguments_file = "arguments.json"
+
+    def _save_info(self, folder: str) -> None:
+        super()._save_info(folder)
+        with open(os.path.join(folder, self.transform_file), "wb") as f:
+            dill.dump(self.test_transform, f)
+
+    @classmethod
+    def _load_info(cls, folder: str) -> Dict[str, Any]:
+        info = super()._load_info(folder)
+        transform_path = os.path.join(folder, cls.transform_file)
+        if not os.path.isfile(transform_path):
+            test_transform = None
+        else:
+            with open(transform_path, "rb") as f:
+                test_transform = dill.load(f)
+        info["test_transform"] = test_transform
+        return info
+
+    def _save_data(self, data_folder: str) -> None:
+        with open(os.path.join(data_folder, self.arguments_file), "w") as f:
+            json.dump(self.arguments, f)
+
+    @classmethod
+    def _load(
+        cls,
+        data_folder: str,
+        info: Dict[str, Any],
+        sample_weights: sample_weights_type,
+    ) -> "CVDataModule":
+        with open(os.path.join(data_folder, cls.arguments_file), "r") as f:
+            kwargs = json.load(f)
+        data = cls(**kwargs)
+        data.test_transform = info["test_transform"]
+        data.prepare(sample_weights)
+        return data
 
 
 @DLDataModule.register("image_folder")
@@ -441,6 +597,7 @@ class ImageFolderData(CVDataModule):
         test_transform_config: Optional[Dict[str, Any]] = None,
         lmdb_config: Optional[Dict[str, Any]] = None,
     ):
+        self.arguments = shallow_copy_dict(get_arguments())
         self.folder = folder
         self.shuffle = shuffle
         self.drop_train_last = drop_train_last
@@ -559,6 +716,7 @@ class InferenceImageFolderData(CVDataModule):
         transform: Optional[Union[str, List[str], Transforms, Callable]] = None,
         transform_config: Optional[Dict[str, Any]] = None,
     ):
+        self.arguments = shallow_copy_dict(get_arguments())
         self.folder = folder
         self.batch_size = batch_size
         self.num_workers = num_workers
@@ -907,7 +1065,12 @@ def prepare_image_folder(
             with open_dtype_file(label_file) as f_:
                 json.dump(type_labels, f_, ensure_ascii=False)
         # lmdb
-        if lmdb_config is None:
+        if lmdb_config is None or lmdb is None:
+            if lmdb_config is not None:
+                print(
+                    f"{WARNING_PREFIX}`lmdb` is not installed, "
+                    "so `lmdb_config` will be ignored"
+                )
             return None
         local_lmdb_config = shallow_copy_dict(lmdb_config)
         local_lmdb_config.setdefault("path", default_lmdb_path(tgt_folder, dtype))
