@@ -3,14 +3,12 @@ import sys
 import json
 import math
 import torch
-import logging
 import argparse
 import torchvision
 import urllib.request
 
 import numpy as np
 import torch.nn as nn
-import matplotlib.pyplot as plt
 import torch.nn.functional as F
 
 from PIL import Image
@@ -29,18 +27,15 @@ from typing import ContextManager
 from zipfile import ZipFile
 from collections import defaultdict
 from collections import OrderedDict
-from onnxruntime import InferenceSession
 from torch.optim import Optimizer
 from cftool.misc import prod
 from cftool.misc import check_requires
 from cftool.misc import shallow_copy_dict
 from cftool.misc import context_error_handler
-from cftool.misc import LoggingMixin
 from cftool.misc import DownloadProgressBar
+from cftool.array import arr_type
 from cftool.array import to_standard
-from cfml.misc.toolkit import show_or_save
 
-from ..types import arr_type
 from ..types import data_type
 from ..types import param_type
 from ..types import np_dict_type
@@ -51,10 +46,35 @@ from ..constants import INPUT_KEY
 from ..constants import INFO_PREFIX
 from ..constants import WARNING_PREFIX
 
+try:
+    import matplotlib.pyplot as plt
+except:
+    plt = None
+try:
+    from onnxruntime import InferenceSession
+except:
+    InferenceSession = None
+try:
+    from cfml.misc.toolkit import show_or_save
+except:
+    show_or_save = None
+
 
 # general
 
 
+def filter_kw(
+    fn: Callable,
+    kwargs: Dict[str, Any],
+    strict: bool = True,
+) -> Dict[str, Any]:
+    kw = {}
+    for k, v in kwargs.items():
+        if check_requires(fn, k, strict):
+            kw[k] = v
+    return kw
+
+
 def check_is_ci() -> bool:
     parser = argparse.ArgumentParser()
     parser.add_argument("--ci", type=int, default=0)
@@ -270,6 +290,10 @@ class WeightsStrategy:
         return 1.0 / (1.0 + np.exp(-x))
 
     def visualize(self, export_path: str = "weights_strategy.png") -> None:
+        if plt is None:
+            raise ValueError(f"{WARNING_PREFIX}`matplotlib` is needed for `visualize`")
+        if show_or_save is None:
+            raise ValueError(f"{WARNING_PREFIX}`carefree-ml` is needed for `visualize`")
         n = 1000
         x = np.linspace(0, 1, n)
         y = self(n, 0)
@@ -280,58 +304,24 @@ class WeightsStrategy:
         show_or_save(export_path)
 
 
-class LoggingMixinWithRank(LoggingMixin):
-    is_rank_0: bool = True
-
-    def set_rank_0(self, value: bool) -> None:
-        self.is_rank_0 = value
-        for v in self.__dict__.values():
-            if isinstance(v, LoggingMixinWithRank):
-                v.set_rank_0(value)
-
-    def _init_logging(
-        self,
-        verbose_level: Optional[int] = 2,
-        trigger: bool = True,
-    ) -> None:
-        if not self.is_rank_0:
-            return None
-        super()._init_logging(verbose_level, trigger)
-
-    def log_msg(
-        self,
-        body: str,
-        prefix: str = "",
-        verbose_level: Optional[int] = 1,
-        msg_level: int = logging.INFO,
-        frame: Any = None,
-    ) -> None:
-        if not self.is_rank_0:
-            return None
-        super().log_msg(body, prefix, verbose_level, msg_level, frame)
-
-    def log_block_msg(
-        self,
-        body: str,
-        prefix: str = "",
-        title: str = "",
-        verbose_level: Optional[int] = 1,
-        msg_level: int = logging.INFO,
-        frame: Any = None,
-    ) -> None:
-        if not self.is_rank_0:
-            return None
-        super().log_block_msg(body, prefix, title, verbose_level, msg_level, frame)
-
-    def log_timing(self) -> None:
-        if not self.is_rank_0:
-            return None
-        return super().log_timing()
-
-
 # dl
 
 
+def get_label_predictions(logits: np.ndarray, threshold: float) -> np.ndarray:
+    # binary classification
+    if logits.shape[-1] == 1:
+        logit_threshold = math.log(threshold / (1.0 - threshold))
+        return (logits > logit_threshold).astype(int)
+    return logits.argmax(1)[..., None]
+
+
+def get_full_logits(logits: np.ndarray) -> np.ndarray:
+    # binary classification
+    if logits.shape[-1] == 1:
+        logits = np.concatenate([-logits, logits], axis=-1)
+    return logits
+
+
 def inject_debug(config: Dict[str, Any]) -> None:
     config["fixed_steps"] = 1
     config["valid_portion"] = 1.0e-4
@@ -788,7 +778,7 @@ class eval_context(mode_context):
         )
 
 
-class Initializer(LoggingMixinWithRank):
+class Initializer:
     """
     Initializer for neural network weights
 
@@ -860,12 +850,10 @@ class Initializer(LoggingMixinWithRank):
                 weight_base[invalid] = param.new_empty(num_invalid).normal_()
                 invalid = get_invalid(weight_base)
         if not success:
-            self.log_msg(
-                f"invalid ratio for truncated normal : {invalid.to(torch.float32).mean():8.6f}, "
-                f"it might cause by too little epoch ({epoch}) or too small tolerance ({tol})",
-                prefix=self.warning_prefix,
-                verbose_level=2,
-                msg_level=logging.WARNING,
+            print(
+                f"{WARNING_PREFIX}invalid ratio for truncated normal : "
+                f"{invalid.to(torch.float32).mean():8.6f}, it might cause by "
+                f"too little epoch ({epoch}) or too small tolerance ({tol})",
             )
         with torch.no_grad():
             param.data.copy_(weight_base.reshape(param.shape))
@@ -908,6 +896,9 @@ class DropNoGradStatesMixin:
 
 class ONNX:
     def __init__(self, onnx_path: str):
+        if InferenceSession is None:
+            msg = "`ONNX` is not available when `onnxruntime` is not installed"
+            raise ValueError(msg)
         self.ort_session = InferenceSession(onnx_path)
         self.output_names = [node.name for node in self.ort_session.get_outputs()]
 
