@@ -12,8 +12,6 @@ from typing import List
 from typing import Tuple
 from typing import Optional
 
-from .losses import GANLoss
-from .losses import GANTarget
 from .discriminators import DiscriminatorBase
 from ..protocol import GaussianGeneratorMixin
 from ....data import CVLoader
@@ -29,6 +27,8 @@ from ....constants import PREDICTIONS_KEY
 from ....misc.toolkit import to_device
 from ....misc.toolkit import mode_context
 from ....misc.toolkit import toggle_optimizer
+from ....losses.gan import GANLoss
+from ....losses.gan import GANTarget
 
 
 class GANMixin(ModelWithCustomSteps, GaussianGeneratorMixin, metaclass=ABCMeta):
