@@ -80,7 +80,7 @@ class Modulator(nn.Module):
 
 def make_grid(size: int, in_dim: int, device: Optional[torch.device] = None) -> Tensor:
     tensors = [torch.linspace(-1.0, 1.0, steps=size, device=device)] * in_dim
-    grid = torch.stack(torch.meshgrid(*tensors), dim=-1)
+    grid = torch.stack(torch.meshgrid(*tensors, indexing="ij"), dim=-1)
     return grid.view(1, -1, grid.shape[-1])
 
 
