@@ -12,14 +12,13 @@ from typing import Union
 from typing import Optional
 
 from .fcnn import FCNN
-from .protocol import MLCoreProtocol
+from .protocol import register_ml_module
 from ..bases import CustomLossBase
 from ...types import losses_type
 from ...types import tensor_dict_type
 from ...protocol import LossProtocol
 from ...protocol import TrainerState
 from ...constants import LOSS_KEY
-from ...constants import INPUT_KEY
 from ...constants import LABEL_KEY
 from ...constants import PREDICTIONS_KEY
 from ..implicit.siren import make_grid
@@ -56,7 +55,7 @@ def _make_ddr_grid(num_samples: int, device: torch.device) -> Tensor:
     return make_grid(num_samples + 2, 1, device)[:, 1:-1]
 
 
-@MLCoreProtocol.register("ddr")
+@register_ml_module("ddr")
 class DDR(CustomLossBase):
     def __init__(
         self,
@@ -77,7 +76,7 @@ class DDR(CustomLossBase):
         predict_cdf: bool = True,
         y_min_max: Optional[Tuple[float, float]] = None,
     ):
-        super().__init__(in_dim, out_dim, num_history)
+        super().__init__()
         self.fcnn = FCNN(
             in_dim,
             out_dim,
@@ -144,21 +143,22 @@ class DDR(CustomLossBase):
         cdf = torch.sigmoid(logit)
         return y_ratio, logit, cdf
 
-    def forward(
+    def forward(  # type: ignore
         self,
-        batch_idx: int,
-        batch: tensor_dict_type,
-        state: Optional[TrainerState] = None,
-        **kwargs: Any,
+        net: Tensor,
+        *,
+        get_cdf: bool = True,
+        get_quantiles: bool = True,
+        tau: Optional[float] = None,
+        y_anchor: Optional[float] = None,
     ) -> tensor_dict_type:
         # prepare
-        net = batch[INPUT_KEY]
         device = net.device
         num_samples = len(net)
         if len(net.shape) > 2:
             net = net.contiguous().view(num_samples, -1)
-        get_quantiles = kwargs.get("get_quantiles", True) and self.predict_quantiles
-        get_cdf = kwargs.get("get_cdf", True) and self.predict_cdf
+        get_quantiles = get_quantiles and self.predict_quantiles
+        get_cdf = get_cdf and self.predict_cdf
         # median forward
         mods = []
         for block in self.fcnn.net:
@@ -172,44 +172,42 @@ class DDR(CustomLossBase):
         y_span = y_max - y_min
         # quantile forward
         if get_quantiles:
-            tau = kwargs.get("tau", None)
             if tau is not None:
-                tau = _expand_element(num_samples, tau, device) * 2.0 - 1.0
+                tau_tensor = _expand_element(num_samples, tau, device) * 2.0 - 1.0
             else:
                 shape = num_samples, self.num_random_samples, 1
                 if self.training:
-                    tau = torch.rand(*shape, device=device) * 2.0 - 1.0
+                    tau_tensor = torch.rand(*shape, device=device) * 2.0 - 1.0
                 else:
-                    tau = _make_ddr_grid(self.num_random_samples, device)
-                    tau = tau.repeat(num_samples, 1, 1)
-            tau.requires_grad_(True)
-            q_increment, quantiles = self._get_quantiles(tau, mods, median)
+                    tau_tensor = _make_ddr_grid(self.num_random_samples, device)
+                    tau_tensor = tau_tensor.repeat(num_samples, 1, 1)
+            tau_tensor.requires_grad_(True)
+            q_increment, quantiles = self._get_quantiles(tau_tensor, mods, median)
             results.update(
                 {
-                    "tau": tau,
+                    "tau": tau_tensor,
                     "q_increment": q_increment,
                     "quantiles": quantiles,
                 }
             )
         # cdf forward
         if get_cdf:
-            y_anchor = kwargs.get("y_anchor", None)
             if y_anchor is not None:
-                y_anchor = _expand_element(num_samples, y_anchor, device)
+                ya_tensor = _expand_element(num_samples, y_anchor, device)
             else:
                 shape = num_samples, self.num_random_samples, 1
                 if self.training:
-                    y_anchor = torch.rand(*shape, device=device) * y_span + y_min
+                    ya_tensor = torch.rand(*shape, device=device) * y_span + y_min
                 else:
                     y_raw_ratio = _make_ddr_grid(self.num_random_samples, device)
                     y_raw_ratio = 0.5 * (y_raw_ratio + 1.0)
-                    y_anchor = (y_raw_ratio * y_span + y_min).repeat(num_samples, 1, 1)
-            y_anchor.requires_grad_(True)
-            y_ratio, logit, cdf = self._get_cdf(y_anchor, median, y_span, mods)
-            pdf = get_gradient(cdf, y_anchor, True, True)  # type: ignore
+                    ya_tensor = (y_raw_ratio * y_span + y_min).repeat(num_samples, 1, 1)
+            ya_tensor.requires_grad_(True)
+            y_ratio, logit, cdf = self._get_cdf(ya_tensor, median, y_span, mods)
+            pdf = get_gradient(cdf, ya_tensor, True, True)  # type: ignore
             results.update(
                 {
-                    "y_anchor": y_anchor,
+                    "y_anchor": ya_tensor,
                     "logit": logit,
                     "cdf": cdf,
                     "pdf": pdf,
