@@ -12,13 +12,12 @@ from typing import Union
 from typing import Optional
 from typing import NamedTuple
 from collections import defaultdict
-from cfdata.tabular.misc import np_int_type
+from cftool.array import arr_type
 
 from ...data import MLLoader
 from ...constants import INPUT_KEY
 from ...misc.toolkit import to_torch
 from ...misc.toolkit import Initializer
-from ...misc.toolkit import LoggingMixinWithRank
 from ...modules.blocks import Lambda
 
 
@@ -73,7 +72,7 @@ class Embedding(nn.Module):
         return self.core(tensor)
 
 
-class Encoder(nn.Module, LoggingMixinWithRank):
+class Encoder(nn.Module):
     def __init__(
         self,
         config: Dict[str, Any],
@@ -83,6 +82,57 @@ class Encoder(nn.Module, LoggingMixinWithRank):
         categorical_columns: List[int],
         loaders: List[MLLoader],
     ):
+        """
+        Encoder to perform differentiable categorical encodings.
+
+        Parameters
+        ----------
+        config (Dict[str, Any]) : configs of the encoder.
+            * embedding_dropout (float) : dropout rate for embedding.
+                * default : 0.2
+            * default_embedding_init_method (str) : default init method for embedding.
+                * default : "truncated_normal"
+            * default_embedding_init_config (Dict[str, Any]) : default init config for embedding.
+                * default : {"mean": 0, "std": 0.02}
+            * use_fast_embedding (bool) : whether to use fast embedding.
+                * default : True
+            * recover_original_dim (bool) : whether to recover original dim when fast embedding is applied.
+                * default : False
+            * unified_embedding_dim (str | int) : unified embedding dim used in fast embedding.
+                * default : "max"
+            * fast_embedding_init_method (str | None) : init method for fast embedding.
+                * default : None
+            * fast_embedding_init_config (Dict[str, Any] | None) : init config for fast embedding.
+                * default : None
+        input_dims (List[int]) : number of different values of each categorical column.
+            * Note that len(input_dims) should be = len(categorical_columns)
+        methods_list (List[Union[str, List[str]]]): list of encoding methods to use
+            for each categorical column.
+            * if List[str] is provided, then multiple encoding methods will be used.
+        method_configs (List[Dict[str, Any]]): configs of the corresponding encoding methods.
+        categorical_columns (List[int]): indices of categorical columns.
+            * Note that len(categorical_columns) should be = len(input_dims)
+
+        Properties
+        ----------
+        _one_hot_indices (List[int]) : indices of categorical columns that are encoded with one-hot.
+            * notice that these indices are indices of `categorical_columns`
+        _embed_indices (List[int]) : indices of categorical columns that are encoded with embedding.
+            * notice that these indices are indices of `categorical_columns`.
+        _embed_dims (Dict[int, int]) : dims of embedding used for each categorical column.
+            * key : index of `categorical_columns`.
+            * value : dimension of the embedding module (self.embeddings[self._embed_indices.index(key)]).
+            * len(_embed_dims) should be = len(_embed_indices).
+        merged_dims (Dict[int, int])
+            * key : index of `categorical_columns`.
+            * value : final encoded dim of the {categorical_columns[key]}th column.
+        embeddings (nn.ModuleList) : list of embedding modules.
+            * ith embedding correspond to the {categorical_columns[self._embed_indices[i]]}th column.
+        one_hot_encoders (nn.ModuleList) : list of one-hot encoder modules.
+            * ith one-hot encoder correspond to the {categorical_columns[self._one_hot_indices[i]]}th column.
+        input_dims (tensor) : same as `input_dims` above, but in torch.tensor format.
+
+        """
         super().__init__()
         self._fe_init_method: Optional[str]
         self._fe_init_config: Optional[Dict[str, Any]]
@@ -95,30 +145,33 @@ class Encoder(nn.Module, LoggingMixinWithRank):
         self.embedding_dim = 0
         dims_tensor = torch.tensor(input_dims, dtype=torch.float32)
         self.register_buffer("input_dims", dims_tensor)
-        self.tgt_columns = np.array(sorted(categorical_columns), np_int_type)
+        self.sorted_categorical_columns = sorted(categorical_columns)
+        self.tgt_columns = np.array(self.sorted_categorical_columns, np.int64)
         self.merged_dims: Dict[int, int] = defaultdict(int)
         self.embeddings = nn.ModuleList()
         self.one_hot_encoders = nn.ModuleList()
         self._one_hot_indices: List[int] = []
         self._embed_indices: List[int] = []
-        self._embed_dims: List[int] = []
+        self._embed_dims: Dict[int, int] = {}
         iterator = zip(input_dims, methods_list, method_configs)
         for i, (in_dim, methods, methods_config) in enumerate(iterator):
             if isinstance(methods, str):
                 methods = [methods]
             self._register(i, in_dim, methods, methods_config)
         # fast embedding
+        self.unified_dim = 0
         if self.use_embedding and self._use_fast_embed:
+            sorted_dims = sorted(self._embed_dims.values())
+            max_dim = sorted_dims[-1]
             if isinstance(self._unified_embed_dim, int):
                 unified_dim = self._unified_embed_dim
             else:
                 if self._unified_embed_dim == "max":
-                    unified_dim = max(self._embed_dims)
+                    unified_dim = max_dim
                 elif self._unified_embed_dim == "mean":
-                    unified_dim = int(round(sum(self._embed_dims) / self.num_embedding))
+                    unified_dim = int(round(sum(sorted_dims) / self.num_embedding))
                 elif self._unified_embed_dim == "median":
                     half_idx = self.num_embedding // 2
-                    sorted_dims = sorted(self._embed_dims)
                     if self.num_embedding % 2 != 0:
                         unified_dim = sorted_dims[half_idx]
                     else:
@@ -130,9 +183,19 @@ class Encoder(nn.Module, LoggingMixinWithRank):
                         f"unified embedding dim '{self._unified_embed_dim}' "
                         "is not recognized"
                     )
-            for i in self._embed_indices:
-                self.merged_dims[i] += unified_dim
-            embedding_dim_sum = unified_dim * self.num_embedding
+            if self._recover_dim and unified_dim < max_dim:
+                raise ValueError(
+                    f"unified embedding dim {unified_dim} is smaller than "
+                    f"max embedding dim {max_dim}, which is not allowed "
+                    f"when `_recover_dim` is set to True"
+                )
+            self.unified_dim = unified_dim
+            embedding_dim_sum = 0
+            for idx in self._embed_indices:
+                original_dim = self._embed_dims[idx]
+                final_dim = original_dim if self._recover_dim else unified_dim
+                self.merged_dims[idx] += final_dim
+                embedding_dim_sum += final_dim
             self.embedding_dim = embedding_dim_sum
             self.merged_dim += embedding_dim_sum
             if self._fe_init_method is None:
@@ -152,11 +215,12 @@ class Encoder(nn.Module, LoggingMixinWithRank):
             embed_dims_cumsum = self.input_dims[self._embed_indices].cumsum(0)[:-1]
             embed_dims_cumsum = embed_dims_cumsum.to(torch.long)
             self.register_buffer("embed_dims_cumsum", embed_dims_cumsum)
-            if not self._recover_dim or len(set(self._embed_dims)) == 1:
+            if not self._recover_dim or len(set(self._embed_dims.values())) == 1:
                 self.register_buffer("recover_indices", None)
             else:
                 recover_indices: List[int] = []
-                for i, dim in enumerate(self._embed_dims):
+                for i, idx in enumerate(sorted(self._embed_dims)):
+                    dim = self._embed_dims[idx]
                     recover_indices.extend(i * unified_dim + j for j in range(dim))
                 recover_indices_tensor = torch.tensor(recover_indices, dtype=torch.long)
                 self.register_buffer("recover_indices", recover_indices_tensor)
@@ -215,6 +279,7 @@ class Encoder(nn.Module, LoggingMixinWithRank):
         if not self.use_embedding:
             embedding = None
         else:
+            use_cache = use_cache and self._use_fast_embed
             if not use_cache:
                 indices = categorical_columns
             else:
@@ -228,6 +293,45 @@ class Encoder(nn.Module, LoggingMixinWithRank):
                 embedding = self.embedding_dropout(embedding)
         return EncodingResult(one_hot, embedding)
 
+    def encode(self, column: arr_type, column_idx: int) -> EncodingResult:
+        try:
+            idx = self.sorted_categorical_columns.index(column_idx)
+        except ValueError:
+            return EncodingResult(None, None)
+        if isinstance(column, np.ndarray):
+            column = to_torch(column)
+        column = column.to(torch.long).view(-1)
+        # one hot
+        if not self.use_one_hot:
+            one_hot = None
+        else:
+            if self._all_one_hot:
+                one_hot_idx = idx
+            else:
+                one_hot_idx = self._one_hot_indices[idx]
+            one_hot = self.one_hot_encoders[one_hot_idx](column)
+        # embedding
+        if not self.use_embedding:
+            embedding = None
+        else:
+            if self._all_embedding:
+                embedding_idx = idx
+            else:
+                embedding_idx = self._embed_indices.index(idx)
+            indices = column.clone()
+            if not self._use_fast_embed:
+                embedding = self.embeddings[embedding_idx](indices)
+            else:
+                if embedding_idx > 0:
+                    indices += self.embed_dims_cumsum[embedding_idx - 1]
+                embedding = self.embeddings[0](indices)
+                if self._recover_dim:
+                    dim = self._embed_dims[idx]
+                    embedding = embedding[..., list(range(dim))]
+            if self.embedding_dropout is not None:
+                embedding = self.embedding_dropout(embedding)
+        return EncodingResult(one_hot, embedding)
+
     def _init_config(self, config: Dict[str, Any]) -> None:
         self.config = config
         self._embed_drop = config.setdefault("embedding_dropout", 0.2)
@@ -285,7 +389,7 @@ class Encoder(nn.Module, LoggingMixinWithRank):
         else:
             raise ValueError(f"embedding dim '{embedding_dim}' is not defined")
         if self._use_fast_embed:
-            self._embed_dims.append(out_dim)
+            self._embed_dims[i] = out_dim
             if self._fe_init_method is None:
                 self._fe_init_method = config.get("init_method")
             if self._fe_init_config is None:
@@ -330,7 +434,7 @@ class Encoder(nn.Module, LoggingMixinWithRank):
     def _embedding(self, indices_columns: torch.Tensor) -> torch.Tensor:
         if self._use_fast_embed:
             embed_mat = self.embeddings[0](indices_columns)
-            embed_mat = embed_mat.view(-1, self.embedding_dim)
+            embed_mat = embed_mat.view(-1, self.num_embedding * self.unified_dim)
             if not self._recover_dim or self.recover_indices is None:
                 return embed_mat
             return embed_mat[..., self.recover_indices]
