@@ -9,7 +9,6 @@ import torch.nn.functional as F
 from abc import abstractmethod
 from abc import ABCMeta
 from torch import Tensor
-from einops import repeat
 from typing import Any
 from typing import Dict
 from typing import List
@@ -1198,7 +1197,8 @@ class PerceiverIO(Module):
         mask: Optional[Tensor] = None,
         out_queries: Optional[Tensor] = None,
     ) -> Tensor:
-        in_latent = repeat(self.in_latent, "n d -> b n d", b=net.shape[0])
+        B = net.shape[0]
+        in_latent = torch.repeat_interleave(self.in_latent[None, ...], B, dim=0)
         for cross_attn, cross_ff, self_attn_blocks in self.layers:
             in_latent = cross_attn(in_latent, net, mask=mask) + in_latent
             in_latent = cross_ff(in_latent) + in_latent
@@ -1206,7 +1206,7 @@ class PerceiverIO(Module):
                 in_latent = self_attn(in_latent) + in_latent
                 in_latent = self_ff(in_latent) + in_latent
         if self.out_latent is not None:
-            out_queries = repeat(self.out_latent, "n d -> b n d", b=net.shape[0])
+            out_queries = torch.repeat_interleave(self.out_latent[None, ...], B, dim=0)
         if out_queries is None:
             return in_latent
         return self.decoder_cross_attn(out_queries, in_latent)
