@@ -272,7 +272,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
         self._write_pipeline_info(export_folder)
         data = getattr(self, "data", None)
         if data is not None:
-            self.data.save(export_folder)
+            self.data.save_info(export_folder)
         # final results
         final_results = None
         try:
@@ -304,6 +304,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
         cls,
         export_folder: str,
         cuda: Optional[str],
+        to_original_device: bool,
         pre_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
         post_callback: Optional[Callable[["DLPipeline", Dict[str, Any]], None]] = None,
     ) -> "DLPipeline":
@@ -314,7 +315,13 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
         config["in_loading"] = True
         m = cls(**config)
         device_info = DeviceInfo(*config_bundle["device_info"])
-        device_info = device_info._replace(cuda=cuda)
+        if not to_original_device:
+            device_info = device_info._replace(cuda=cuda)
+        elif cuda is not None:
+            print(
+                f"{WARNING_PREFIX}`to_original_device` is set to True, so "
+                f"`cuda={cuda}` will be ignored"
+            )
         m.device_info = device_info
         if post_callback is not None:
             post_callback(m, config_bundle)
@@ -471,6 +478,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
         export_folder: str,
         *,
         cuda: Optional[Union[int, str]] = None,
+        to_original_device: bool = False,
         compress: bool = True,
         states_callback: states_callback_type = None,
         pre_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
@@ -485,11 +493,12 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
                 m = cls._load_infrastructure(
                     export_folder,
                     None if cuda is None else str(cuda),
+                    to_original_device,
                     pre_callback,
                     post_callback,
                 )
                 try:
-                    data_info = DataModule.load(export_folder)
+                    data_info = DataModule.load_info(export_folder)
                 except Exception as err:
                     print(
                         f"{WARNING_PREFIX}error occurred when trying to load "
